#include <io.h>
#include <stdio.h>
#include <direct.h>
#include <windows.h>
#include <sys/stat.h>
#include "mpc\in_mpc.h"
#include "mpc\requant.h"
#include "mpc\huffsv46.h"
#include "mpc\huffsv7.h"
#include "mpc\bitstream.h"
#include "mpc\mpc_dec.h"
#include "mpc\minimax.h"

#ifndef _MAX_PATH
#define _MAX_PATH   260
#endif

unsigned int min   = 0;
unsigned int max   = -1;
unsigned int minbr = 0;
unsigned int maxbr = -1;
unsigned int tper  = 0;

void printtime ( unsigned int frame1, unsigned int frame2, double sfreq )
{
    double tmp;
    tmp = frame1 * 1152 / (sfreq/1000);
    unsigned int msecs1 = (unsigned int)tmp;
    tmp = frame2 * 1152 / (sfreq/1000);
    unsigned int msecs2 = (unsigned int)tmp;

    printf ( "%02u:%02u.%04u - %02u:%02u.%04u: ", (msecs1/1000)/60, (msecs1/1000)%60, msecs1%1000, (msecs2/1000)/60, (msecs2/1000)%60, msecs2%1000 );
}

int view_mpc_bits ( const char *fn )
{
    MPC_decoder *theDec = NULL;
    Reader* reader = NULL;
    StreamInfo *Info = NULL;

    FILE *fp = fopen ( fn, "rb" );
    if ( !fp ) {
        printf ( "%s: file open failed\n\n", fn );
        return 1;
    }

    reader = new Reader_file ( fp );
    Info = new StreamInfo ();
    if ( Info->ReadStreamInfo (reader) != 0 ) {
        printf ( "%s: unrecognized format\n\n", fn );
        if ( reader ) delete reader;
        if ( Info ) delete Info;
        return 1;
    }

    theDec = new MPC_decoder ( reader );
    theDec->RESET_Globals   ();
    theDec->RESET_Synthesis ();
    theDec->SetStreamInfo ( Info );
    theDec->FileInit ();

    printf ( "%s:\n", fn );

    unsigned int lastbits  = 0;
    unsigned int lastframe = 0;

    while ( 1 ) {
        unsigned int RING = theDec->Zaehler;
        unsigned int FrameBitCnt = theDec->BitsRead();

        int valid_samples = theDec->DECODE ( NULL );
        theDec->UpdateBuffer ( RING );

        unsigned int msecs = theDec->DecodedFrames * 1152 / (int)(Info->simple.SampleFreq/1000);
        if ( theDec->FrameWasValid == 0 ) {
            printtime ( theDec->DecodedFrames-1, theDec->DecodedFrames, Info->simple.SampleFreq );
            printf ( "BROKEN FILE......\n" );
            break;
        }
        if ( valid_samples <= 0 ) break;

        double tmp = (double)tper * Info->simple.SampleFreq / 1152000.0;
        unsigned int tpframes = (unsigned int)tmp;

        if ( theDec->DecodedFrames - lastframe >= tpframes ) {
            unsigned int bits   = theDec->BitsRead() - lastbits;
            unsigned int frames = theDec->DecodedFrames - lastframe;
            double       msecs  = frames * 1152.0 / (Info->simple.SampleFreq/1000);

            double br = bits / msecs;
            unsigned int frbits = theDec->BitsRead() - FrameBitCnt;

            if ( (frbits >= min && frbits <= max) && (br >= minbr && br <= maxbr) ) {
                printtime ( lastframe, theDec->DecodedFrames, Info->simple.SampleFreq );
                printf ( "%6.1f %7u\n", br, bits );
            }

            lastbits  = theDec->BitsRead();
            lastframe = theDec->DecodedFrames;
        }
    }

    if ( Info ) delete Info;
    if ( reader ) delete reader;
    if ( theDec ) delete theDec;

    printf ( "\n" );

    return 0;
}

void setpriority ()
{
    HANDLE  pid = GetCurrentProcess ();
    HANDLE  tid = GetCurrentThread  ();

    SetPriorityClass ( pid, IDLE_PRIORITY_CLASS );
    SetThreadPriority ( tid, THREAD_PRIORITY_IDLE );
}

void usage ()
{
    printf ( "Usage: mpcbits [options] <filename>\n"
             "Displays bit usage in MPC file.\n"
             "Default time period is smallest possible, one frame (~26ms)\n"
             "\n"
             "Options:\n"
             " -min <x>   : only show time periods with over <x> bits\n"
             " -max <x>   : only show time periods with under <x> bits\n"
             " -minbr <x> : only show time periods with bitrate higher than <x>\n"
             " -maxbr <x> : only show time periods with bitrate lower than <x>\n"
             " -time <x>  : use time period x for bitrate calculations\n"
             "              bitrate is given in kbps, time in ms\n"
           );
}

int view_files ( const char *fn )
{
    char                olddir[_MAX_PATH];
    char                temp[_MAX_PATH];
    char                drive[_MAX_DRIVE];
    char                dir[_MAX_DIR];
    char                fname[_MAX_FNAME];
    char                ext[_MAX_EXT];
    struct _finddata_t  fd;
    long                hFile;

    _getcwd ( olddir, _MAX_PATH );
    _splitpath ( fn, drive, dir, fname, ext );
    _makepath ( temp, drive, dir, NULL, NULL );
    _chdir ( temp );

    if ( (hFile = _findfirst (fn, &fd)) != -1L ) {
        if ( !(fd.attrib & _A_SUBDIR) && !(fd.attrib & _A_HIDDEN) ) {
            view_mpc_bits ( fd.name );
        }

        while ( _findnext (hFile, &fd) == 0 ) {
            if ( fd.attrib & _A_SUBDIR || fd.attrib & _A_HIDDEN )
                continue;

            view_mpc_bits ( fd.name );
        }

        _findclose ( hFile );
    }

    _chdir ( olddir );

    return 0;
}

int main ( int argc, char **argv )
{
    if ( argc < 2 ) {
        usage ();
        return 0;
    }

    int i;
    for ( i = 1; i < argc-1; i++ ) {
        if ( argv[i][0] == '-' ) {
            if ( strcmp ( argv[i], "-min" ) == 0 ) {
                min = atoi ( argv[++i] );
            } else
            if ( strcmp ( argv[i], "-max" ) == 0 ) {
                max = atoi ( argv[++i] );
            } else
            if ( strcmp ( argv[i], "-minbr" ) == 0 ) {
                minbr = atoi ( argv[++i] );
            } else
            if ( strcmp ( argv[i], "-maxbr" ) == 0 ) {
                maxbr = atoi ( argv[++i] );
            } else
            if ( strcmp ( argv[i], "-time" ) == 0 ) {
                tper = atoi ( argv[++i] );
            }
        }
    }

    setpriority ();

    for ( i = 1; i < argc; i++ ) {
        if ( argv[i][0] == '-' ) continue;
        if ( argv[i-1][0] == '-' ) continue;

        view_files ( argv[i] );
    }

    return 0;
}
